; ---------------------------------------------------------------------------
; Interlace pictures viewer (step 1) / (c) SyX 2011
; ---------------------------------------------------------------------------

    include "firmware.i"
    include "hardware.i"
    include "macros.i"

SOUND_CLICK                 EQU $4000

; Load address for the frames
ORG_P00                     EQU $0080
ORG_P01                     EQU $4000
ORG_I00                     EQU $8020
ORG_I01                     EQU $C000

    ORG  $8000

start
    ; Get active disk drive and save it for after
    LD   HL,(AMSDOS_RESERVED_AREA)
    LD   A,(HL)                 ; Active disk drive for Amsdos (0 = A / 1 = B)
    LD   (disk_drive + 1),A
restart
    LD   C,$FF                  ; Disable ROMs
    LD   HL,main_program        ; HL = Start program address
    CALL MC_START_PROGRAM       ; Run program

main_program
    CALL TXT_VDU_DISABLE        ; Disable VDU
    CALL KL_ROM_WALK            ; Enable ROMS
    CALL TXT_VDU_ENABLE         ; Enable VDU

    ; Set display size for the firmware
    CALL MC_WAIT_FLYBACK
    LD   HL,crtc_firmware_values
    LD   A,6
    CALL set_crtc_values

    ; Set active diskdrive for the Amsdos (the same used for load this program)
disk_drive
    LD   A,0
    LD   HL,(AMSDOS_RESERVED_AREA)
    LD   (HL),A

launch_console
    CALL execute_console

    ; Load files
    ; * Palette *
    LD   HL,pal_file                            ; HL = address filename
    LD   B,end_pal_file - pal_file              ; B = length filename
    CALL CAS_IN_OPEN
    LD   A,C
    LD   (len_palette),A                        ; Save palette length
    OR   A
    LD   HL,fin_palette_lace
    SBC  HL,BC                                  ; HL = load address - length palette
    CALL CAS_IN_DIRECT
    JR   NC,launch_console
    CALL CAS_IN_CLOSE 

    ; Cargamos P00 en la página 0
    LD   HL,gfx_file                            ; HL = address filename
    LD   B,end_gfx_file - gfx_file              ; B = length filename
    CALL CAS_IN_OPEN
    LD   HL,ORG_P00                             ; HL = load address
    CALL CAS_IN_DIRECT
    CALL CAS_IN_CLOSE 

    ; Cargamos P01 en la página 1
    LD   A,'1'
    LD   (gfx_file + 11),A                      ; Change P00 -> P01
    LD   HL,gfx_file                            ; HL = address filename
    LD   B,end_gfx_file - gfx_file              ; B = length filename
    CALL CAS_IN_OPEN
    LD   HL,ORG_P01                             ; HL = load address
    CALL CAS_IN_DIRECT
    CALL CAS_IN_CLOSE 

    ; Cargamos I01 en la página 3
    LD   A,'I'
    LD   (gfx_file + 9),A                       ; Change P01 -> I01
    LD   HL,gfx_file                            ; HL = address filename
    LD   B,end_gfx_file - gfx_file              ; B = length filename
    CALL CAS_IN_OPEN
    LD   HL,ORG_I01                             ; HL = load address
    CALL CAS_IN_DIRECT
    CALL CAS_IN_CLOSE 

    ; Activamos la página 4
    LD   BC,GATE_ARRAY + $C4
    OUT  (C),C

    ; Cargamos I00 en la página 4
    LD   A,'0'
    LD   (gfx_file + 11),A                      ; Change I01 -> I00
    LD   HL,gfx_file                            ; HL = address filename
    LD   B,end_gfx_file - gfx_file              ; B = length filename
    CALL CAS_IN_OPEN
    LD   HL,$4000                               ; HL = load address
    CALL CAS_IN_DIRECT
    CALL CAS_IN_CLOSE 

    ; Copiamos la fase 2 a un hueco en la página 3
    LD   HL,fase_02
    LD   DE,$FCE0
    LD   BC,fin_fase_02 - fase_02
    LDIR

    ; Y saltamos a la fase 2 en la página 3 (FX Entrelazado :P)
    JP   $FCE0

; ---------------------------------------------------------------------------
fase_02
    incbin "loader_fase_02.bin"

    ; Palette buffer is at the end of fase_02
palette_lace
    DEFS 16
fin_palette_lace

len_palette
    DEFB 0

fin_fase_02

; ---------------------------------------------------------------------------
; Show the files with mode 5 pictures from the disk and let you choose the 
; file to view.
; ---------------------------------------------------------------------------
execute_console
    LD   HL,console_text
    CALL print_text

    CALL print_disk_catalog

    LD   HL,prompt_text
    CALL print_text

    ; Initialize the filename strings
    LD   HL,pal_file
    LD   (HL)," "
    LD   DE,pal_file + 1
    LD   BC,7
    LDIR

    LD   HL,gfx_file + 9
    LD   (HL),"P"
    INC  HL
;    LD   (HL),"0"
    INC  HL
    LD   (HL),"0"


    ; Copy sound_data to central ram
    LD   HL,sound_data
    LD   DE,SOUND_CLICK
    LD   BC,end_sound_data - sound_data
    LDIR

    ; Input filename
    LD   DE,pal_file
    LD   B,9
.loop_key_input
    CALL KM_WAIT_KEY
    CALL make_sound
    CP   $0D
    JR   Z,.end_loop_key_input
    CP   $7F
    JR   NZ,.print_char
    LD   A,B
    CP   9
    JR   Z,.loop_key_input
    LD   HL,delete_char_text
    CALL print_text
    INC  B
    DEC  DE
    LD   A," "
    LD   (DE),A
    JR   .loop_key_input       
.print_char
    LD   L,A
    LD   A,B
    CP   1
    JR   Z,.loop_key_input
    LD   A,L
    CALL TXT_OUTPUT
    LD   (DE),A
    INC  DE
    LD   HL,cursor_text
    CALL print_text
    DJNZ .loop_key_input
.end_loop_key_input
    LD   A," "
    CALL TXT_OUTPUT

    ; Copy filename pal string to gfx string
    LD   HL,pal_file
    LD   DE,gfx_file
    LD   BC,8
    LDIR

    ; Y ponemos las tintas a 0
    LD   HL,apagon_text
    CALL print_text

    RET

; ---------------------------------------------------------------------------
; Get and print the disk catalog
; ---------------------------------------------------------------------------
print_disk_catalog
    ; Get disk catalog
    CALL TXT_VDU_DISABLE            ; Disable VDU
    XOR  A
    LD   DE,catalog_buffer
    LD   (DE),A                     ; Set flag catalog buffer clear
    CALL CAS_CATALOG
    CALL TXT_VDU_ENABLE             ; Enable VDU

    ; Print filenames with Interlace GFXs
    LD   IX,catalog_buffer
.loop_decode_cat
    LD   A,(IX + 0)
    OR   A
    RET  Z
    ; Test for PLC extension
    LD   A,(IX + 9)
    CP   "P"
    JR   NZ,.next_file
    LD   A,(IX + 10)
    CP   "L"
    JR   NZ,.next_file
    LD   A,(IX + 11)
    CP   "C"
    JR   NZ,.next_file
    ; Interlace file found
    LD   (IX + 0),$20
    LD   (IX + 9),$20
    LD   (IX + 10),$FF
    LD   D,IXH
    LD   E,IXL
    EX   DE,HL
    CALL print_text
.next_file
    LD   BC,14
    ADD  IX,BC
    JR   .loop_decode_cat

; ---------------------------------------------------------------------------
; Print a string finished in $FF using the firmware
; Entry:
;    HL : Pointer to the string
; ---------------------------------------------------------------------------
print_text
    LD   A,(HL)
.loop_print_text
    CALL TXT_OUTPUT
    INC  HL
    LD   A,(HL)
.continue_print_text
    CP   $FF
    JR   NZ,.loop_print_text
    RET

; ---------------------------------------------------------------------------
; Make a click, to give feedback when a key is pressed
; ---------------------------------------------------------------------------
make_sound
    PUSH AF
    PUSH BC
    PUSH DE
    PUSH HL
    PUSH IX

    LD   HL,SOUND_CLICK     ; sound_data in central ram
    CALL SOUND_QUEUE

    POP  IX
    POP  HL
    POP  DE
    POP  BC
    POP  AF
    RET

sound_data
    DEFB 2,0,0
    DEFW 478
    DEFB 0,15
    DEFW 1
end_sound_data

; ---------------------------------------------------------------------------
; Set CRTC values
set_crtc_values
    LD   B,>CRTC_WRITE
.bucle_define_pantalla
    OUTI
    INC  B
    INC  B
    OUTI
    DEC  A
    JR   NZ,.bucle_define_pantalla
    RET

crtc_firmware_values                ; Taken from the firmware
    DEFB REG_01,$28
    DEFB REG_06,$19
    DEFB REG_02,$2E
    DEFB REG_07,$1E
    DEFB REG_04,$26
    DEFB REG_05,$00
;    DEFB REG_0C,VRAM_P3
;    DEFB REG_0D,$00

; ---------------------------------------------------------------------------
; Text data for the console screen
console_text
    DEFB $1C,$00,$00,$00    ; INK 0,0,0
    DEFB $1C,$01,$1A,$1A    ; INK 1,26,26
    DEFB $1C,$02,$12,$12    ; INK 2,18,18
    DEFB $1C,$03,$19,$19    ; INK 3,25,25
    DEFB $1D,$00,$00        ; BORDER 0,0
    DEFB $04,$01            ; MODE 1
    DEFB $1F,$01,$01        ; LOCATE 1,1
    DEFB $0F,$01            ; PEN 1
    DEFB $18                ; VIDEO INVERSE ON
    DEFB "INTERLACE VIEWER"
    DEFB $12                ; CLEAR FROM CURSOR TO RIGHT
    DEFB $1F,$1F,$01        ; LOCATE 1,30
    DEFB $A4," SyX 2011"
    DEFB $18                ; VIDEO INVERSE OFF
    DEFB $0F,$03            ; PEN 3
    DEFB $FF

prompt_text
    DEFB $1F,$01,$19        ; LOCATE 1,25
    DEFB $0F,$02            ; PEN 2
    DEFB "syx@arnold:~$ "
    DEFB $0F,$01            ; PEN 1
    DEFB "_",$08
    DEFB $FF

delete_char_text
    DEFB " ",$08," ",$08,$08
cursor_text
    DEFB "_",$08,$FF

apagon_text
    DEFB $1C,$01,$00,$00    ; INK 1,0,0
    DEFB $1C,$02,$00,$00    ; INK 2,0,0
    DEFB $1C,$03,$00,$00    ; INK 3,0,0
    DEFB $FF

; ---------------------------------------------------------------------------
; Files with the GFX + PAL
pal_file
    DEFB "SYX@@DAD.PLC"
end_pal_file

gfx_file
    DEFB "DAD@@SYX.P00"
end_gfx_file

; ---------------------------------------------------------------------------
; Format catalog buffer:
; $FF, file name (8), file extension (3), size in KBs (2),
; $FF, file name (8), file extension (3), size in KBs (2),
; ...
; $00
catalog_buffer

; ---------------------------------------------------------------------------
    END
