; ---------------------------------------------------------------------------
; Interlace pictures viewer (step 2) / (c) SyX 2011
; ---------------------------------------------------------------------------

R8_LACE_DEFAULT             EQU 0

    include "firmware.i"
    include "hardware.i"
    include "macros.i"

; Load address for the frames
ORG_P00                     EQU $0080
ORG_P01                     EQU $4000
ORG_I00                     EQU $8020
ORG_I01                     EQU $C000

; CRTC values
WIDTH_CRTC                  EQU 48                              ; 48 caracteres en mode 1 (192 pixels en modo 0)
OFFSET_HORIZONTAL           EQU 26 + WIDTH_CRTC / 2    
OFFSET_CRTC_FRAME1          EQU 64                              ; Offset para evitar discontinuidades en la ruptura
OFFSET_CRTC_FRAME2          EQU 16                              ; Offset para evitar discontinuidades en la ruptura

; CRTC values for the PAL/SECAM overscan
HEIGHT_CRTC_PAL             EQU 34                              ; 34 caracteres (272 líneas)
OFFSET_VERTICAL_PAL         EQU 16 + (HEIGHT_CRTC_PAL * 8) / 14
VERTICAL_TOTAL_PAL          EQU 38
VERTICAL_TOTAL_ADJUST_PAL   EQU 0

; CRTC values for the NTSC overscan
HEIGHT_CRTC_NTSC            EQU 30                              ; 30 caracteres (240 líneas)
OFFSET_VERTICAL_NTSC        EQU 30
VERTICAL_TOTAL_NTSC         EQU 31
VERTICAL_TOTAL_ADJUST_NTSC  EQU 6

; Constantes de las teclas
KEY_UP                      EQU 0
KEY_DOWN                    EQU 1
KEY_LEFT                    EQU 2
KEY_RIGHT                   EQU 3
KEY_MODE                    EQU 4
KEY_R8                      EQU 5
KEY_SWAP                    EQU 6
KEY_EXIT                    EQU 7

; Dirección de la paleta en fase_01 ($8000)
DIR_PALETA_F1               EQU $83B4

    ORG  $FCE0

inicio
    ; Tomamos el control del sistema
    DI

    ; Páginamos la página 5
    LD   BC,GATE_ARRAY + $C5
    OUT  (C),C

    ; Y copiamos la página 2 a ella (firmware y código del programa)
    LD   HL,$8000
    LD   DE,$4000
    LD   BC,$4000
    LDIR

    ; Copiamos el vector de interrupciones del sistema y la pila
    LD   HL,($0038)
    LD   (sm_restore_vint + 1),HL
    LD   HL,$C9FB
    LD   ($0038),HL
    LD   (sm_firmware_sp + 1),SP
    LD   SP,$F800                   ; Situamos la pila en uno de los huecos del overscan
    EI

    ; Activamos la página 4
    LD   BC,GATE_ARRAY + $C4
    OUT  (C),C

    ; Copiamos I00 a la página 2
    LD   HL,$4000
    LD   DE,ORG_I00
    LD   BC,$3FE0
    LDIR

    ; Definimos las dimensiones del CRTC
    CALL espera_refresco
    LD   HL,crtc_lace_pal_values
    LD   A,6 ;8
    CALL set_crtc_values

    ; Actualizamos los centrados
    CALL actualiza_centrados_pal

    ; Offset vertical por defecto
    LD   A,OFFSET_VERTICAL_PAL
    LD   (offset_vertical),A

    ; ¿Entrelazado "hardware" activado por defecto?
    IF R8_LACE_DEFAULT
        LD   BC,CRTC_SELECT + REG_08
        OUT  (C),C
        LD   BC,CRTC_WRITE + $01
        OUT  (C),C
    ENDIF 

    ; Establecemos el modo de pantalla
    LD   A,(paleta + 16)         ; Recuperamos el número de colores de la imagen
    LD   BC,GATE_ARRAY + MODE_Y_ROM + ROM_OFF + MODE_1
.sera_mode_2
    CP   2
    JR   NZ,.sera_mode_0
    INC  C
    JR   .cambia_modo
.sera_mode_0
    CP   16
    JR   NZ,.sera_mode_1
    DEC  C
.sera_mode_1
.cambia_modo
    OUT  (C),C

    ; Y ponemos la paleta de la pantalla    
    ; A = Número de elementos de la paleta
    LD   HL,paleta + 16          ; Puntero al último elemento de la paleta
establece_paleta
    LD   B,>GATE_ARRAY
.bucle_establece_paleta
    OUT  (C),A
    INC  B
    OUTD
    DEC  A
    JP   P,.bucle_establece_paleta

    HALT
    HALT
    HALT

loop_fx_rutine
    CALL espera_refresco

sm_pon_frame_01
    CALL pon_frame_impar

    HALT
    HALT
    HALT

    CALL comprueba_teclado

sm_pon_frame_02
    CALL pon_frame_par

    HALT
    HALT
    HALT

    JR   loop_fx_rutine
; ---------------------------------------------------------------------------
pon_frame_impar
    LD   BC,CRTC_SELECT + REG_0C
    OUT  (C),C
    LD   BC,CRTC_WRITE + VRAM_P2 + VRAM32
    OUT  (C),C
    LD   BC,CRTC_SELECT + REG_0D
    OUT  (C),C
    LD   BC,CRTC_WRITE + OFFSET_CRTC_FRAME2
    OUT  (C),C
    RET

pon_frame_par
    LD   BC,CRTC_SELECT + REG_0C
    OUT  (C),C
    LD   BC,CRTC_WRITE + VRAM_P0 + VRAM32
    OUT  (C),C
    LD   BC,CRTC_SELECT + REG_0D
    OUT  (C),C
    LD   BC,CRTC_WRITE + OFFSET_CRTC_FRAME1
    OUT  (C),C
    RET

; ---------------------------------------------------------------------------
; Rutina que restaura el firmware y regresa al visor
; ---------------------------------------------------------------------------
end_fx_rutine
    ; Empezamos a restaurar el firmware
    DI

    ; Páginamos la página 5
    LD   BC,GATE_ARRAY + $C5
    OUT  (C),C

    ; Y restauramos la copia del firmware que teníamos allí
    LD   HL,$4000
    LD   DE,$8000
    LD   BC,$4000
    LDIR

    ; Guardamos los valores de centrado actualizados
    LD   A,(centrado_horizontal)
    LD   (DIR_PALETA_F1 - 3),A
    LD   A,(centrado_vertical)
    LD   (DIR_PALETA_F1 - 2),A

    ; Restauramos el vector de interrupciones del sistema y la pila
sm_restore_vint
    LD   HL,$000
    LD   ($0038),HL
sm_firmware_sp
    LD   SP,$0000

    ; Despaginamos la ram expandida
    LD   BC,GATE_ARRAY + $C0
    OUT  (C),C

    ; Restore alternative registers used by the firmware
    EXX
    LD   BC,$7F8D
    EXX  
    EX   AF,AF'
    XOR  A
    EX   AF,AF'

    EI

    ; Y regresamos al visor
    JP   $8007

; ---------------------------------------------------------------------------
; Set CRTC values
; ---------------------------------------------------------------------------
set_crtc_values
    LD   B,>CRTC_WRITE
.bucle_define_pantalla
    OUTI
    INC  B
    INC  B
    OUTI
    DEC  A
    JR   NZ,.bucle_define_pantalla
    RET

; ---------------------------------------------------------------------------
; Realizamos la lectura del teclado y actuamos en consecuencia
; ---------------------------------------------------------------------------
comprueba_teclado
sm_ignora_teclado
    LD   A,$00
    OR   A
    JR   Z,haz_lectura
    ; Comprobamos si hay teclas pulsadas
    CALL any_key_press
    JP   C,fin_comprobaciones
    LD   A,(sm_ignora_teclado + 1)
    XOR  $FF
    LD   (sm_ignora_teclado + 1),A
    JP   fin_comprobaciones

haz_lectura
    CALL lee_teclado
    BIT  KEY_EXIT,A
    JP   NZ,end_fx_rutine

comprueba_espacio
    BIT  KEY_MODE,A
    JP   Z,comprueba_intro
    CALL marca_ignora_teclado
sm_ntsc
    LD   A,0
    XOR  $FF
    LD   (sm_ntsc + 1),A
    OR   A
    JR   Z,pon_pal
pon_ntsc
    LD   HL,crtc_lace_ntsc_values
    LD   A,6
    CALL set_crtc_values
    LD   A,OFFSET_VERTICAL_NTSC
    LD   (offset_vertical),A
    CALL actualiza_centrados_ntsc
    JP   hubo_cambio_en_crtc
pon_pal
    LD   HL,crtc_lace_pal_values
    LD   A,6
    CALL set_crtc_values
    LD   A,OFFSET_VERTICAL_PAL
    LD   (offset_vertical),A
    CALL actualiza_centrados_pal
    JP   hubo_cambio_en_crtc

comprueba_intro
    BIT  KEY_R8,A
    JR   Z,comprueba_copy
    CALL marca_ignora_teclado
sm_r8
    LD   A,0
    XOR  $FF
    LD   (sm_r8 + 1),A
    OR   A
    JR   Z,sin_r8
con_r8
    LD   A,1
sin_r8
    ; Activamos el entrelazado en el CRTC 0 del David
    LD   BC,CRTC_SELECT + REG_08
    OUT  (C),C
    INC  B
    OUT  (C),A
    JP   hubo_cambio_en_crtc

comprueba_copy
    BIT  KEY_SWAP,A
    JP   Z,comprueba_arriba
    CALL marca_ignora_teclado
sm_swap_frames
    LD   A,0
    XOR  $FF
    LD   (sm_swap_frames + 1),A
    OR   A
    JR   Z,pon_impar_par
pon_par_impar
    LD   HL,pon_frame_par
    LD   (sm_pon_frame_01 + 1),HL
    LD   HL,pon_frame_impar
    LD   (sm_pon_frame_02 + 1),HL
    JP   fin_comprobaciones
pon_impar_par
    LD   HL,pon_frame_impar
    LD   (sm_pon_frame_01 + 1),HL
    LD   HL,pon_frame_par
    LD   (sm_pon_frame_02 + 1),HL
    JP   hubo_cambio_en_crtc

comprueba_arriba
    BIT  KEY_UP,A
    JR   Z,comprueba_abajo
    CALL marca_ignora_teclado
    ; Movemos la pantalla hacía arriba
    LD   A,(centrado_vertical)
sm_limite_arriba
    CP   2  ; 3 Cuelgue en NTSC
    JP   Z,hubo_cambio_en_crtc
    INC  A
    LD   (centrado_vertical),A
    LD   B,A
    LD   A,(offset_vertical)
    ADD  B
    WRITE_CRTC_REG REG_07,A
    JP   hubo_cambio_en_crtc

comprueba_abajo
    BIT  KEY_DOWN,A
    JR   Z,comprueba_izquierda
    CALL marca_ignora_teclado
    ; Movemos la pantalla hacía abajo
    LD   A,(centrado_vertical)
    CP   $FD    ; -3 en complemento a 2
    JP   Z,hubo_cambio_en_crtc
    DEC  A
    LD   (centrado_vertical),A
    LD   B,A
    LD   A,(offset_vertical)
    ADD  B
    WRITE_CRTC_REG REG_07,A
    JP   hubo_cambio_en_crtc

comprueba_izquierda
    BIT  KEY_LEFT,A
    JR   Z,comprueba_derecha
    CALL marca_ignora_teclado
    ; Movemos la pantalla hacía la izquierda
    LD   A,(centrado_horizontal)
    CP   3
    JP   Z,hubo_cambio_en_crtc
    INC  A
    LD   (centrado_horizontal),A
    LD   B,OFFSET_HORIZONTAL
    ADD  B
    WRITE_CRTC_REG REG_02,A
    JP   hubo_cambio_en_crtc

comprueba_derecha
    BIT  KEY_RIGHT,A
    JR   Z,fin_comprobaciones
    CALL marca_ignora_teclado
    ; Movemos la pantalla hacía la derecha
    LD   A,(centrado_horizontal)
    CP   $FD    ; -3 en complemento a 2
    JP   Z,hubo_cambio_en_crtc
    DEC  A
    LD   (centrado_horizontal),A
    LD   B,OFFSET_HORIZONTAL
    ADD  B
    WRITE_CRTC_REG REG_02,A
    JP   hubo_cambio_en_crtc

fin_comprobaciones
    CALL espera_refresco
hubo_cambio_en_crtc

    RET

marca_ignora_teclado
    LD   A,(sm_ignora_teclado + 1)
    XOR  $FF
    LD   (sm_ignora_teclado + 1),A
    CALL espera_refresco
    RET

; ---------------------------------------------------------------------------
; any_key_press: Comprueba si existe alguna tecla pulsada y lo indica en el Carry Flag
; ---------------------------------------------------------------------------
any_key_press
    CALL inicializa_psg             ; 1, 2, 3 y 4 (Destruye BC)

    LD   C,%01000000                ; 5 Escribir la línea de la matriz del teclado a comprobar en los bits 3-0 del PPI Port C
.bucle_any_key_press
    CALL lee_byte_psg_en_b          ; 6 y 7 Leemos del PSG la línea de la matriz del teclado
    LD   A,B
    CPL                             ; Si una tecla ó botón de joystick es pulsado su bit estará a 1

    AND  A
    JR   NZ,.si_any_key_press       ; Había teclas pulsadas???
    INC  C

    LD   A,%01001010                ; Se comprobaron todas las líneas del teclado???
    CP   C
    JR   NZ,.bucle_any_key_press
    JR   .no_any_key_press
.si_any_key_press
    SCF
    JR   .cont_any_key_press
.no_any_key_press
    AND  A                          ; SCF & CCF
.cont_any_key_press
    CALL restaura_psg               ; 9 y 10 (Destruye BC)
    RET

; ---------------------------------------------------------------------------
; Lee el estado de los cursores, espacio e intro
; Salida: 
;    A: Contiene las teclas pulsadas
;       0: Arriba
;       1: Abajo
;       2: Izquierda
;       3: Derecha
;       4: Space (NTSC/PAL)
;       5: Intro (R8 on/off)
;       6: Copy (Intercambia el cambio de los frames)
;       7: Escape (Regresamos al visor)
; ---------------------------------------------------------------------------
lee_teclado
    CALL inicializa_psg         ; 1º, 2º, 3º y 4º (Destruye BC)

    ; Inicializamos la variable con las teclas pulsadas
    XOR A

    ; Comprobamos Cursores Up, Down y Right
    LD   C,%01000000            ; Línea de la matriz del teclado a comprobar (bits 3-0 del PPI Port C)
    CALL lee_byte_psg_en_b      ; B contiene el valor de la línea
.test_cursor_up
    BIT  CURSOR_UP,B
    JR   NZ,.test_cursor_down
    SET  KEY_UP,A
.test_cursor_down
    BIT  CURSOR_DOWN,B
    JR   NZ,.test_cursor_right
    SET  KEY_DOWN,A
.test_cursor_right
    BIT  CURSOR_RIGHT,B
    JR   NZ,.test_cursor_left
    SET  KEY_RIGHT,A

    ; Comprobamos Cursor Left y Copy
.test_cursor_left
    INC  C                      ; C = %01000001
    CALL lee_byte_psg_en_b      ; B contiene el valor de la línea
    BIT  CURSOR_LEFT,B
    JR   NZ,.test_copy
    SET  KEY_LEFT,A
.test_copy
    BIT  KEY_COPY,B
    JR   NZ,.test_return
    SET  KEY_SWAP,A

    ; Comprobamos Return
.test_return
    INC  C                      ; C = %01000010
    CALL lee_byte_psg_en_b      ; B contiene el valor de la línea
    BIT  KEY_RETURN,B
    JR   NZ,.test_space
    SET  KEY_R8,A

    ; Comprobamos Espacio
.test_space
    LD   C,%01000101            ; Línea de la matriz del teclado a comprobar (bits 3-0 del PPI Port C)
    CALL lee_byte_psg_en_b      ; B contiene el valor de la línea
    BIT  KEY_SPACE,B
    JR   NZ,.test_escape
    SET  KEY_MODE,A

    ; Comprobamos Escape
.test_escape
    LD   C,%01001000            ; Línea de la matriz del teclado a comprobar (bits 3-0 del PPI Port C)
    CALL lee_byte_psg_en_b      ; B contiene el valor de la línea
    BIT  KEY_ESC,B
    JR   NZ,.fin_lee_teclado
    SET  KEY_EXIT,A

.fin_lee_teclado
    CALL restaura_psg           ; 9º y 10º (Destruye BC)

    RET

; ---------------------------------------------------------------------------
; Actualizamos los centrados
actualiza_centrados_pal
    LD   A,(sm_pal_horizontal + 1)
    LD   D,A
    LD   A,(sm_pal_vertical + 1)
    LD   E,A
    JR   actualiza_centrados
actualiza_centrados_ntsc
    LD   A,(sm_ntsc_horizontal + 1)
    LD   D,A
    LD   A,(sm_ntsc_vertical + 1)
    LD   E,A
actualiza_centrados
    ; Horizontal
    LD   A,(centrado_horizontal)
    ADD  D
    LD   BC,CRTC_SELECT + REG_02
    OUT  (C),C
    INC  B
    OUT  (C),A
    ; Vertical
    LD   A,(centrado_vertical)
    ADD  E
    LD   BC,CRTC_SELECT + REG_07
    OUT  (C),C
    INC  B
    OUT  (C),A
    RET

; ---------------------------------------------------------------------------
; Rutinas cortas para reducir el tamaño del programa
inicializa_psg
    INIT_PSG_FOR_SCANKEYS           ; 1, 2, 3 y 4 (Destruye BC)
    RET

lee_byte_psg_en_b
    LD   B,>PPI_C
    OUT  (C),C                  
    READ_PSG_RB                     ; B contiene el valor de la línea
    RET


restaura_psg
    RESTORE_PSG_OF_SCANKEYS         ; 9 y 10 (Destruye BC)
    RET

espera_refresco
    WAIT_VBL
    RET

; ---------------------------------------------------------------------------
crtc_lace_pal_values
    DEFB REG_01,WIDTH_CRTC
    DEFB REG_06,HEIGHT_CRTC_PAL
sm_pal_horizontal
    DEFB REG_02,OFFSET_HORIZONTAL
sm_pal_vertical
    DEFB REG_07,OFFSET_VERTICAL_PAL
    DEFB REG_04,VERTICAL_TOTAL_PAL
    DEFB REG_05,VERTICAL_TOTAL_ADJUST_PAL

; ---------------------------------------------------------------------------
crtc_lace_ntsc_values
    DEFB REG_01,WIDTH_CRTC
    DEFB REG_06,HEIGHT_CRTC_NTSC
sm_ntsc_horizontal
    DEFB REG_02,OFFSET_HORIZONTAL
sm_ntsc_vertical
    DEFB REG_07,OFFSET_VERTICAL_NTSC
    DEFB REG_04,VERTICAL_TOTAL_NTSC
    DEFB REG_05,VERTICAL_TOTAL_ADJUST_NTSC

; ---------------------------------------------------------------------------
centrado_horizontal
    DEFB 0

centrado_vertical
    DEFB 0

offset_vertical
    DEFB OFFSET_VERTICAL_PAL

; ---------------------------------------------------------------------------
paleta

; ---------------------------------------------------------------------------
    END
